#include <stdio.h>
#include <string.h>

#include "lnm/common.h"
#include "lsm/store.h"

#include "lander.h"

const char lander_key_charset[] =
    "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";

void *lander_gctx_init() { return calloc(1, sizeof(lander_gctx)); }

lnm_err lander_ctx_init(void **c_ctx, void *gctx) {
  lander_ctx *ctx = calloc(1, sizeof(lander_ctx));

  if (ctx == NULL) {
    return lnm_err_failed_alloc;
  }

  *c_ctx = ctx;

  return lnm_err_ok;
}

void lander_ctx_reset(lander_ctx *ctx) {
  if (ctx->entry.read != NULL) {
    if (ctx->write) {
      lsm_write_close(ctx->entry.write);
    } else {
      lsm_read_close(ctx->entry.read);
    }

    ctx->entry.read = NULL;
    ctx->write = false;
  }
}

void lander_ctx_free(lander_ctx *ctx) { free(ctx); }

void lander_header_to_attr(lnm_http_loop_ctx *ctx, const char *header_name,
                           lander_attr_type attr_type) {
  lander_ctx *c_ctx = ctx->c;

  const char *header_value;
  size_t header_value_len;

  if (lnm_http_req_header_get_s(&header_value, &header_value_len, &ctx->req,
                                header_name) == lnm_err_ok) {
    lsm_str *value;
    lsm_str_init_copy_n(&value, (char *)header_value, header_value_len);

    lsm_write_attr_insert(c_ctx->entry.write, attr_type, value);
  }
}

void lander_attr_to_header(lnm_http_loop_ctx *ctx, lander_attr_type attr_type,
                           lnm_http_header header_type) {
  lander_ctx *c_ctx = ctx->c;
  const lsm_str *value;
  lsm_error res;

  if (c_ctx->write) {
    res = lsm_write_attr_get(&value, c_ctx->entry.write, attr_type);
  } else {
    res = lsm_read_attr_get(&value, c_ctx->entry.read, attr_type);
  }

  if (res == lsm_error_ok) {
    lnm_http_res_add_header_len(&ctx->res, header_type,
                                (char *)lsm_str_ptr(value), lsm_str_len(value),
                                false);
  }
}
