#include <string.h>

#include "lnm/http/loop.h"
#include "lnm/log.h"
#include "lnm/loop.h"

#include "lander.h"
#include "lsm/store.h"

lnm_http_step_err lander_stream_body_to_entry(lnm_http_conn *conn) {
  lnm_http_loop_ctx *ctx = conn->ctx;
  lander_ctx *c_ctx = ctx->c;

  uint64_t to_append = LNM_MIN(conn->r.size - conn->r.read,
                               ctx->req.body.expected_len -
                                   lsm_write_data_len(c_ctx->entry.write));

  lsm_str *data;
  lsm_str_init_copy_n(&data, (char *)&conn->r.buf[conn->r.read], to_append);
  lsm_write_data_append(c_ctx->entry.write, data);

  conn->r.read += to_append;

  lsm_str_free(data);

  return lsm_write_data_len(c_ctx->entry.write) == ctx->req.body.expected_len
             ? lnm_http_step_err_done
             : lnm_http_step_err_io_needed;
}

lnm_http_step_err lander_commit_entry(lnm_http_conn *conn) {
  lnm_http_loop_ctx *ctx = conn->ctx;
  lander_ctx *c_ctx = ctx->c;

  lsm_write_commit(c_ctx->entry.write);

  return lnm_http_step_err_done;
}

lnm_http_step_err lander_auth_or_placeholder(lnm_http_conn *conn) {
  lnm_http_loop_ctx *ctx = conn->ctx;
  lnm_http_loop_gctx *gctx = ctx->g;
  lander_gctx *c_gctx = gctx->c;
  lander_ctx *c_ctx = ctx->c;

  const char *key_s;
  size_t key_len = lnm_http_req_route_segment(&key_s, &ctx->req, "key");

  // Only predefined keys can be placeholders
  if (key_len == 0) {
    return lnm_http_loop_step_auth(conn);
  }

  lsm_str *key;
  lsm_str_init_copy_n(&key, key_s, key_len);

  lsm_error res = lsm_store_open_write(&c_ctx->entry.write, c_gctx->store, key);

  lsm_str_free(key);

  switch (res) {
  case lsm_error_ok: {
    c_ctx->write = true;

    lander_entry_type t;

    // If the entry is a placeholder, the request is always authenticated
    if ((lsm_write_attr_get_uint8_t(&t, c_ctx->entry.write,
                                    lander_attr_type_entry_type) ==
         lsm_error_ok) &&
        (t == lander_entry_type_placeholder)) {
      return lnm_http_step_err_done;
    } else {
      return lnm_http_loop_step_auth(conn);
    }
  } break;
  case lsm_error_not_found:
    return lnm_http_loop_step_auth(conn);
    break;
  default:
    ctx->res.status = lnm_http_status_internal_server_error;
    return lnm_http_step_err_res;
  }

  return lnm_http_step_err_done;
}
